"""Base wrapper class for accessing ethereum smart contracts."""

from typing import Any

from eth_utils import is_address, to_checksum_address
from web3 import Web3
from web3.providers.base import BaseProvider

from .tx_params import TxParams


class Validator:
    """Base class for validating inputs to methods."""

    def __init__(self, provider: BaseProvider, contract_address: str):
        """Initialize the instance."""

    def assert_valid(
        self, method_name: str, parameter_name: str, argument_value: Any
    ):
        """Raise an exception if method input is not valid.

        :param method_name: Name of the method whose input is to be validated.
        :param parameter_name: Name of the parameter whose input is to be
            validated.
        :param argument_value: Value of argument to parameter to be validated.
        """


class ContractMethod:
    """Base class for wrapping an Ethereum smart contract method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        validator: Validator = None,
    ):
        """Instantiate the object.

        :param provider: Instance of :class:`web3.providers.base.BaseProvider`
        :param contract_address: Where the contract has been deployed to.
        :param validator: Used to validate method inputs.
        """
        self._web3_eth = Web3(provider).eth  # pylint: disable=no-member
        if validator is None:
            validator = Validator(provider, contract_address)
        self.validator = validator

    @staticmethod
    def validate_and_checksum_address(address: str):
        """Validate the given address, and return it's checksum address."""
        if not is_address(address):
            raise TypeError("Invalid address provided: {}".format(address))
        return to_checksum_address(address)

    def normalize_tx_params(self, tx_params) -> TxParams:
        """Normalize and return the given transaction parameters."""
        if not tx_params:
            tx_params = TxParams()
        if not tx_params.from_:
            tx_params.from_ = self._web3_eth.defaultAccount or (
                self._web3_eth.accounts[0]
                if len(self._web3_eth.accounts) > 0
                else None
            )
        if tx_params.from_:
            tx_params.from_ = self.validate_and_checksum_address(
                tx_params.from_
            )
        return tx_params



"""Generated wrapper for EthBalanceChecker Solidity contract."""

# pylint: disable=too-many-arguments

import json
from typing import (  # pylint: disable=unused-import
    Any,
    List,
    Optional,
    Tuple,
    Union,
)

from eth_utils import to_checksum_address
from mypy_extensions import TypedDict  # pylint: disable=unused-import
from hexbytes import HexBytes
from web3 import Web3
from web3.contract import ContractFunction
from web3.datastructures import AttributeDict
from web3.providers.base import BaseProvider

from zero_ex.contract_wrappers.bases import ContractMethod, Validator
from zero_ex.contract_wrappers.tx_params import TxParams


# Try to import a custom validator class definition; if there isn't one,
# declare one that we can instantiate for the default argument to the
# constructor for EthBalanceChecker below.
try:
    # both mypy and pylint complain about what we're doing here, but this
    # works just fine, so their messages have been disabled here.
    from . import (  # type: ignore # pylint: disable=import-self
        EthBalanceCheckerValidator,
    )
except ImportError:

    class EthBalanceCheckerValidator(  # type: ignore
        Validator
    ):
        """No-op input validator."""


class GetEthBalancesMethod(ContractMethod):
    """Various interfaces to the getEthBalances method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, addresses: List[str]):
        """Validate the inputs to the getEthBalances method."""
        self.validator.assert_valid(
            method_name="getEthBalances",
            parameter_name="addresses",
            argument_value=addresses,
        )
        return addresses

    def call(
        self, addresses: List[str], tx_params: Optional[TxParams] = None
    ) -> List[int]:
        """Execute underlying contract method via eth_call.

        Batch fetches ETH balances

        :param addresses: Array of addresses.
        :param tx_params: transaction parameters
        :returns: Array of ETH balances.
        """
        (addresses) = self.validate_and_normalize_inputs(addresses)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(addresses).call(tx_params.as_dict())

    def send_transaction(
        self, addresses: List[str], tx_params: Optional[TxParams] = None
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Batch fetches ETH balances

        :param addresses: Array of addresses.
        :param tx_params: transaction parameters
        :returns: Array of ETH balances.
        """
        (addresses) = self.validate_and_normalize_inputs(addresses)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(addresses).transact(tx_params.as_dict())

    def estimate_gas(
        self, addresses: List[str], tx_params: Optional[TxParams] = None
    ) -> int:
        """Estimate gas consumption of method call."""
        (addresses) = self.validate_and_normalize_inputs(addresses)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(addresses).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self, addresses: List[str], tx_params: Optional[TxParams] = None
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (addresses) = self.validate_and_normalize_inputs(addresses)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(addresses).buildTransaction(
            tx_params.as_dict()
        )


# pylint: disable=too-many-public-methods,too-many-instance-attributes
class EthBalanceChecker:
    """Wrapper class for EthBalanceChecker Solidity contract."""

    get_eth_balances: GetEthBalancesMethod
    """Constructor-initialized instance of
    :class:`GetEthBalancesMethod`.
    """

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        validator: EthBalanceCheckerValidator = None,
    ):
        """Get an instance of wrapper for smart contract.

        :param provider: instance of :class:`web3.providers.base.BaseProvider`
        :param contract_address: where the contract has been deployed
        :param validator: for validation of method inputs.
        """
        self.contract_address = contract_address

        if not validator:
            validator = EthBalanceCheckerValidator(provider, contract_address)

        self._web3_eth = Web3(  # type: ignore # pylint: disable=no-member
            provider
        ).eth

        functions = self._web3_eth.contract(
            address=to_checksum_address(contract_address),
            abi=EthBalanceChecker.abi(),
        ).functions

        self.get_eth_balances = GetEthBalancesMethod(
            provider, contract_address, functions.getEthBalances, validator
        )

    @staticmethod
    def abi():
        """Return the ABI to the underlying contract."""
        return json.loads(
            '[{"constant":true,"inputs":[{"name":"addresses","type":"address[]"}],"name":"getEthBalances","outputs":[{"name":"","type":"uint256[]"}],"payable":false,"stateMutability":"view","type":"function"}]'  # noqa: E501 (line-too-long)
        )


# pylint: disable=too-many-lines



"""Validate inputs to the Exchange contract."""

from typing import Any

from web3.providers.base import BaseProvider

from zero_ex import json_schemas

from ..bases import Validator
from .types import order_to_jsdict


class ExchangeValidator(Validator):
    """Validate inputs to Exchange methods."""

    def __init__(self, provider: BaseProvider, contract_address: str):
        """Initialize the class."""
        super().__init__(provider, contract_address)
        self.contract_address = contract_address

    def assert_valid(
        self, method_name: str, parameter_name: str, argument_value: Any
    ) -> None:
        """Raise an exception if method input is not valid.

        :param method_name: Name of the method whose input is to be validated.
        :param parameter_name: Name of the parameter whose input is to be
            validated.
        :param argument_value: Value of argument to parameter to be validated.
        """
        if parameter_name == "order":
            json_schemas.assert_valid(
                order_to_jsdict(argument_value, self.contract_address),
                "/orderSchema",
            )

        if parameter_name == "orders":
            for order in argument_value:
                json_schemas.assert_valid(
                    order_to_jsdict(order, self.contract_address),
                    "/orderSchema",
                )



"""Generated wrapper for DutchAuction Solidity contract."""

# pylint: disable=too-many-arguments

import json
from typing import (  # pylint: disable=unused-import
    Any,
    List,
    Optional,
    Tuple,
    Union,
)

from eth_utils import to_checksum_address
from mypy_extensions import TypedDict  # pylint: disable=unused-import
from hexbytes import HexBytes
from web3 import Web3
from web3.contract import ContractFunction
from web3.datastructures import AttributeDict
from web3.providers.base import BaseProvider

from zero_ex.contract_wrappers.bases import ContractMethod, Validator
from zero_ex.contract_wrappers.tx_params import TxParams


# Try to import a custom validator class definition; if there isn't one,
# declare one that we can instantiate for the default argument to the
# constructor for DutchAuction below.
try:
    # both mypy and pylint complain about what we're doing here, but this
    # works just fine, so their messages have been disabled here.
    from . import (  # type: ignore # pylint: disable=import-self
        DutchAuctionValidator,
    )
except ImportError:

    class DutchAuctionValidator(  # type: ignore
        Validator
    ):
        """No-op input validator."""


class Tuple0xbb41e5b3(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    makerAssetFilledAmount: int

    takerAssetFilledAmount: int

    makerFeePaid: int

    takerFeePaid: int


class Tuple0x054ca44e(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    left: Tuple0xbb41e5b3

    right: Tuple0xbb41e5b3

    leftMakerAssetSpreadAmount: int


class Tuple0x260219a2(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    makerAddress: str

    takerAddress: str

    feeRecipientAddress: str

    senderAddress: str

    makerAssetAmount: int

    takerAssetAmount: int

    makerFee: int

    takerFee: int

    expirationTimeSeconds: int

    salt: int

    makerAssetData: bytes

    takerAssetData: bytes


class Tuple0xdc58a88c(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    beginTimeSeconds: int

    endTimeSeconds: int

    beginAmount: int

    endAmount: int

    currentAmount: int

    currentTimeSeconds: int


class GetAuctionDetailsMethod(ContractMethod):
    """Various interfaces to the getAuctionDetails method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, order: Tuple0x260219a2):
        """Validate the inputs to the getAuctionDetails method."""
        self.validator.assert_valid(
            method_name="getAuctionDetails",
            parameter_name="order",
            argument_value=order,
        )
        return order

    def call(
        self, order: Tuple0x260219a2, tx_params: Optional[TxParams] = None
    ) -> Union[Tuple0xdc58a88c, Union[HexBytes, bytes]]:
        """Execute underlying contract method via eth_call.

        Calculates the Auction Details for the given order

        :param order: The sell order
        :param tx_params: transaction parameters
        :returns: the return value of the underlying method.
        """
        (order) = self.validate_and_normalize_inputs(order)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order).call(tx_params.as_dict())

    def send_transaction(
        self, order: Tuple0x260219a2, tx_params: Optional[TxParams] = None
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Calculates the Auction Details for the given order

        :param order: The sell order
        :param tx_params: transaction parameters
        """
        (order) = self.validate_and_normalize_inputs(order)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order).transact(tx_params.as_dict())

    def estimate_gas(
        self, order: Tuple0x260219a2, tx_params: Optional[TxParams] = None
    ) -> int:
        """Estimate gas consumption of method call."""
        (order) = self.validate_and_normalize_inputs(order)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order).estimateGas(tx_params.as_dict())

    def build_transaction(
        self, order: Tuple0x260219a2, tx_params: Optional[TxParams] = None
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (order) = self.validate_and_normalize_inputs(order)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order).buildTransaction(
            tx_params.as_dict()
        )


class MatchOrdersMethod(ContractMethod):
    """Various interfaces to the matchOrders method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(
        self,
        buy_order: Tuple0x260219a2,
        sell_order: Tuple0x260219a2,
        buy_signature: bytes,
        sell_signature: bytes,
    ):
        """Validate the inputs to the matchOrders method."""
        self.validator.assert_valid(
            method_name="matchOrders",
            parameter_name="buyOrder",
            argument_value=buy_order,
        )
        self.validator.assert_valid(
            method_name="matchOrders",
            parameter_name="sellOrder",
            argument_value=sell_order,
        )
        self.validator.assert_valid(
            method_name="matchOrders",
            parameter_name="buySignature",
            argument_value=buy_signature,
        )
        buy_signature = bytes.fromhex(buy_signature.decode("utf-8"))
        self.validator.assert_valid(
            method_name="matchOrders",
            parameter_name="sellSignature",
            argument_value=sell_signature,
        )
        sell_signature = bytes.fromhex(sell_signature.decode("utf-8"))
        return (buy_order, sell_order, buy_signature, sell_signature)

    def call(
        self,
        buy_order: Tuple0x260219a2,
        sell_order: Tuple0x260219a2,
        buy_signature: bytes,
        sell_signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> Union[Tuple0x054ca44e, Union[HexBytes, bytes]]:
        """Execute underlying contract method via eth_call.

        Matches the buy and sell orders at an amount given the following: the
        current block time, the auction start time and the auction begin
        amount. The sell order is a an order at the lowest amount at the end of
        the auction. Excess from the match is transferred to the seller. Over
        time the price moves from beginAmount to endAmount given the current
        block.timestamp. sellOrder.expiryTimeSeconds is the end time of the
        auction. sellOrder.takerAssetAmount is the end amount of the auction
        (lowest possible amount). sellOrder.makerAssetData is the ABI encoded
        Asset Proxy data with the following data appended
        buyOrder.makerAssetData is the buyers bid on the auction, must meet the
        amount for the current block timestamp (uint256 beginTimeSeconds,
        uint256 beginAmount). This function reverts in the following scenarios:
        * Auction has not started (auctionDetails.currentTimeSeconds <
        auctionDetails.beginTimeSeconds) * Auction has expired
        (auctionDetails.endTimeSeconds < auctionDetails.currentTimeSeconds) *
        Amount is invalid: Buy order amount is too low
        (buyOrder.makerAssetAmount < auctionDetails.currentAmount) * Amount is
        invalid: Invalid begin amount (auctionDetails.beginAmount >
        auctionDetails.endAmount) * Any failure in the 0x Match Orders

        :param buyOrder: The Buyer's order. This order is for the current
            expected price of the auction.
        :param buySignature: Proof that order was created by the buyer.
        :param sellOrder: The Seller's order. This order is for the lowest
            amount (at the end of the auction).
        :param sellSignature: Proof that order was created by the seller.
        :param tx_params: transaction parameters
        :returns: the return value of the underlying method.
        """
        (
            buy_order,
            sell_order,
            buy_signature,
            sell_signature,
        ) = self.validate_and_normalize_inputs(
            buy_order, sell_order, buy_signature, sell_signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            buy_order, sell_order, buy_signature, sell_signature
        ).call(tx_params.as_dict())

    def send_transaction(
        self,
        buy_order: Tuple0x260219a2,
        sell_order: Tuple0x260219a2,
        buy_signature: bytes,
        sell_signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Matches the buy and sell orders at an amount given the following: the
        current block time, the auction start time and the auction begin
        amount. The sell order is a an order at the lowest amount at the end of
        the auction. Excess from the match is transferred to the seller. Over
        time the price moves from beginAmount to endAmount given the current
        block.timestamp. sellOrder.expiryTimeSeconds is the end time of the
        auction. sellOrder.takerAssetAmount is the end amount of the auction
        (lowest possible amount). sellOrder.makerAssetData is the ABI encoded
        Asset Proxy data with the following data appended
        buyOrder.makerAssetData is the buyers bid on the auction, must meet the
        amount for the current block timestamp (uint256 beginTimeSeconds,
        uint256 beginAmount). This function reverts in the following scenarios:
        * Auction has not started (auctionDetails.currentTimeSeconds <
        auctionDetails.beginTimeSeconds) * Auction has expired
        (auctionDetails.endTimeSeconds < auctionDetails.currentTimeSeconds) *
        Amount is invalid: Buy order amount is too low
        (buyOrder.makerAssetAmount < auctionDetails.currentAmount) * Amount is
        invalid: Invalid begin amount (auctionDetails.beginAmount >
        auctionDetails.endAmount) * Any failure in the 0x Match Orders

        :param buyOrder: The Buyer's order. This order is for the current
            expected price of the auction.
        :param buySignature: Proof that order was created by the buyer.
        :param sellOrder: The Seller's order. This order is for the lowest
            amount (at the end of the auction).
        :param sellSignature: Proof that order was created by the seller.
        :param tx_params: transaction parameters
        """
        (
            buy_order,
            sell_order,
            buy_signature,
            sell_signature,
        ) = self.validate_and_normalize_inputs(
            buy_order, sell_order, buy_signature, sell_signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            buy_order, sell_order, buy_signature, sell_signature
        ).transact(tx_params.as_dict())

    def estimate_gas(
        self,
        buy_order: Tuple0x260219a2,
        sell_order: Tuple0x260219a2,
        buy_signature: bytes,
        sell_signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (
            buy_order,
            sell_order,
            buy_signature,
            sell_signature,
        ) = self.validate_and_normalize_inputs(
            buy_order, sell_order, buy_signature, sell_signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            buy_order, sell_order, buy_signature, sell_signature
        ).estimateGas(tx_params.as_dict())

    def build_transaction(
        self,
        buy_order: Tuple0x260219a2,
        sell_order: Tuple0x260219a2,
        buy_signature: bytes,
        sell_signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (
            buy_order,
            sell_order,
            buy_signature,
            sell_signature,
        ) = self.validate_and_normalize_inputs(
            buy_order, sell_order, buy_signature, sell_signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            buy_order, sell_order, buy_signature, sell_signature
        ).buildTransaction(tx_params.as_dict())


# pylint: disable=too-many-public-methods,too-many-instance-attributes
class DutchAuction:
    """Wrapper class for DutchAuction Solidity contract.

    All method parameters of type `bytes`:code: should be encoded as UTF-8,
    which can be accomplished via `str.encode("utf_8")`:code:.
    """

    get_auction_details: GetAuctionDetailsMethod
    """Constructor-initialized instance of
    :class:`GetAuctionDetailsMethod`.
    """

    match_orders: MatchOrdersMethod
    """Constructor-initialized instance of
    :class:`MatchOrdersMethod`.
    """

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        validator: DutchAuctionValidator = None,
    ):
        """Get an instance of wrapper for smart contract.

        :param provider: instance of :class:`web3.providers.base.BaseProvider`
        :param contract_address: where the contract has been deployed
        :param validator: for validation of method inputs.
        """
        self.contract_address = contract_address

        if not validator:
            validator = DutchAuctionValidator(provider, contract_address)

        self._web3_eth = Web3(  # type: ignore # pylint: disable=no-member
            provider
        ).eth

        functions = self._web3_eth.contract(
            address=to_checksum_address(contract_address),
            abi=DutchAuction.abi(),
        ).functions

        self.get_auction_details = GetAuctionDetailsMethod(
            provider, contract_address, functions.getAuctionDetails, validator
        )

        self.match_orders = MatchOrdersMethod(
            provider, contract_address, functions.matchOrders, validator
        )

    @staticmethod
    def abi():
        """Return the ABI to the underlying contract."""
        return json.loads(
            '[{"constant":false,"inputs":[{"components":[{"name":"makerAddress","type":"address"},{"name":"takerAddress","type":"address"},{"name":"feeRecipientAddress","type":"address"},{"name":"senderAddress","type":"address"},{"name":"makerAssetAmount","type":"uint256"},{"name":"takerAssetAmount","type":"uint256"},{"name":"makerFee","type":"uint256"},{"name":"takerFee","type":"uint256"},{"name":"expirationTimeSeconds","type":"uint256"},{"name":"salt","type":"uint256"},{"name":"makerAssetData","type":"bytes"},{"name":"takerAssetData","type":"bytes"}],"name":"order","type":"tuple"}],"name":"getAuctionDetails","outputs":[{"components":[{"name":"beginTimeSeconds","type":"uint256"},{"name":"endTimeSeconds","type":"uint256"},{"name":"beginAmount","type":"uint256"},{"name":"endAmount","type":"uint256"},{"name":"currentAmount","type":"uint256"},{"name":"currentTimeSeconds","type":"uint256"}],"name":"auctionDetails","type":"tuple"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":false,"inputs":[{"components":[{"name":"makerAddress","type":"address"},{"name":"takerAddress","type":"address"},{"name":"feeRecipientAddress","type":"address"},{"name":"senderAddress","type":"address"},{"name":"makerAssetAmount","type":"uint256"},{"name":"takerAssetAmount","type":"uint256"},{"name":"makerFee","type":"uint256"},{"name":"takerFee","type":"uint256"},{"name":"expirationTimeSeconds","type":"uint256"},{"name":"salt","type":"uint256"},{"name":"makerAssetData","type":"bytes"},{"name":"takerAssetData","type":"bytes"}],"name":"buyOrder","type":"tuple"},{"components":[{"name":"makerAddress","type":"address"},{"name":"takerAddress","type":"address"},{"name":"feeRecipientAddress","type":"address"},{"name":"senderAddress","type":"address"},{"name":"makerAssetAmount","type":"uint256"},{"name":"takerAssetAmount","type":"uint256"},{"name":"makerFee","type":"uint256"},{"name":"takerFee","type":"uint256"},{"name":"expirationTimeSeconds","type":"uint256"},{"name":"salt","type":"uint256"},{"name":"makerAssetData","type":"bytes"},{"name":"takerAssetData","type":"bytes"}],"name":"sellOrder","type":"tuple"},{"name":"buySignature","type":"bytes"},{"name":"sellSignature","type":"bytes"}],"name":"matchOrders","outputs":[{"components":[{"components":[{"name":"makerAssetFilledAmount","type":"uint256"},{"name":"takerAssetFilledAmount","type":"uint256"},{"name":"makerFeePaid","type":"uint256"},{"name":"takerFeePaid","type":"uint256"}],"name":"left","type":"tuple"},{"components":[{"name":"makerAssetFilledAmount","type":"uint256"},{"name":"takerAssetFilledAmount","type":"uint256"},{"name":"makerFeePaid","type":"uint256"},{"name":"takerFeePaid","type":"uint256"}],"name":"right","type":"tuple"},{"name":"leftMakerAssetSpreadAmount","type":"uint256"}],"name":"matchedFillResults","type":"tuple"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"inputs":[{"name":"_exchange","type":"address"}],"payable":false,"stateMutability":"nonpayable","type":"constructor"}]'  # noqa: E501 (line-too-long)
        )


# pylint: disable=too-many-lines



"""Generated wrapper for Coordinator Solidity contract."""

# pylint: disable=too-many-arguments

import json
from typing import (  # pylint: disable=unused-import
    Any,
    List,
    Optional,
    Tuple,
    Union,
)

from eth_utils import to_checksum_address
from mypy_extensions import TypedDict  # pylint: disable=unused-import
from hexbytes import HexBytes
from web3 import Web3
from web3.contract import ContractFunction
from web3.datastructures import AttributeDict
from web3.providers.base import BaseProvider

from zero_ex.contract_wrappers.bases import ContractMethod, Validator
from zero_ex.contract_wrappers.tx_params import TxParams


# Try to import a custom validator class definition; if there isn't one,
# declare one that we can instantiate for the default argument to the
# constructor for Coordinator below.
try:
    # both mypy and pylint complain about what we're doing here, but this
    # works just fine, so their messages have been disabled here.
    from . import (  # type: ignore # pylint: disable=import-self
        CoordinatorValidator,
    )
except ImportError:

    class CoordinatorValidator(  # type: ignore
        Validator
    ):
        """No-op input validator."""


class Tuple0xa1ae4ec4(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    salt: int

    signerAddress: str

    data: bytes


class Tuple0x621bd3c4(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    txOrigin: str

    transactionHash: bytes

    transactionSignature: bytes

    approvalExpirationTimeSeconds: int


class Tuple0x260219a2(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    makerAddress: str

    takerAddress: str

    feeRecipientAddress: str

    senderAddress: str

    makerAssetAmount: int

    takerAssetAmount: int

    makerFee: int

    takerFee: int

    expirationTimeSeconds: int

    salt: int

    makerAssetData: bytes

    takerAssetData: bytes


class GetSignerAddressMethod(ContractMethod):
    """Various interfaces to the getSignerAddress method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, _hash: bytes, signature: bytes):
        """Validate the inputs to the getSignerAddress method."""
        self.validator.assert_valid(
            method_name="getSignerAddress",
            parameter_name="hash",
            argument_value=_hash,
        )
        self.validator.assert_valid(
            method_name="getSignerAddress",
            parameter_name="signature",
            argument_value=signature,
        )
        signature = bytes.fromhex(signature.decode("utf-8"))
        return (_hash, signature)

    def call(
        self,
        _hash: bytes,
        signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> str:
        """Execute underlying contract method via eth_call.

        Recovers the address of a signer given a hash and signature.

        :param hash: Any 32 byte hash.
        :param signature: Proof that the hash has been signed by signer.
        :param tx_params: transaction parameters

        """
        (_hash, signature) = self.validate_and_normalize_inputs(
            _hash, signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(_hash, signature).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self,
        _hash: bytes,
        signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Recovers the address of a signer given a hash and signature.

        :param hash: Any 32 byte hash.
        :param signature: Proof that the hash has been signed by signer.
        :param tx_params: transaction parameters

        """
        (_hash, signature) = self.validate_and_normalize_inputs(
            _hash, signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(_hash, signature).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self,
        _hash: bytes,
        signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (_hash, signature) = self.validate_and_normalize_inputs(
            _hash, signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(_hash, signature).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self,
        _hash: bytes,
        signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (_hash, signature) = self.validate_and_normalize_inputs(
            _hash, signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(_hash, signature).buildTransaction(
            tx_params.as_dict()
        )


class GetTransactionHashMethod(ContractMethod):
    """Various interfaces to the getTransactionHash method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, transaction: Tuple0xa1ae4ec4):
        """Validate the inputs to the getTransactionHash method."""
        self.validator.assert_valid(
            method_name="getTransactionHash",
            parameter_name="transaction",
            argument_value=transaction,
        )
        return transaction

    def call(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_params: Optional[TxParams] = None,
    ) -> bytes:
        """Execute underlying contract method via eth_call.

        Calculates the EIP712 hash of a 0x transaction using the domain
        separator of the Exchange contract.

        :param transaction: 0x transaction containing salt, signerAddress, and
            data.
        :param tx_params: transaction parameters
        :returns: EIP712 hash of the transaction with the domain separator of
            this contract.
        """
        (transaction) = self.validate_and_normalize_inputs(transaction)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(transaction).call(tx_params.as_dict())

    def send_transaction(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Calculates the EIP712 hash of a 0x transaction using the domain
        separator of the Exchange contract.

        :param transaction: 0x transaction containing salt, signerAddress, and
            data.
        :param tx_params: transaction parameters
        :returns: EIP712 hash of the transaction with the domain separator of
            this contract.
        """
        (transaction) = self.validate_and_normalize_inputs(transaction)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(transaction).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (transaction) = self.validate_and_normalize_inputs(transaction)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(transaction).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (transaction) = self.validate_and_normalize_inputs(transaction)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(transaction).buildTransaction(
            tx_params.as_dict()
        )


class GetCoordinatorApprovalHashMethod(ContractMethod):
    """Various interfaces to the getCoordinatorApprovalHash method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, approval: Tuple0x621bd3c4):
        """Validate the inputs to the getCoordinatorApprovalHash method."""
        self.validator.assert_valid(
            method_name="getCoordinatorApprovalHash",
            parameter_name="approval",
            argument_value=approval,
        )
        return approval

    def call(
        self, approval: Tuple0x621bd3c4, tx_params: Optional[TxParams] = None
    ) -> bytes:
        """Execute underlying contract method via eth_call.

        Calculated the EIP712 hash of the Coordinator approval mesasage using
        the domain separator of this contract.

        :param approval: Coordinator approval message containing the
            transaction hash, transaction signature, and expiration of the
            approval.
        :param tx_params: transaction parameters
        :returns: EIP712 hash of the Coordinator approval message with the
            domain separator of this contract.
        """
        (approval) = self.validate_and_normalize_inputs(approval)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(approval).call(tx_params.as_dict())

    def send_transaction(
        self, approval: Tuple0x621bd3c4, tx_params: Optional[TxParams] = None
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Calculated the EIP712 hash of the Coordinator approval mesasage using
        the domain separator of this contract.

        :param approval: Coordinator approval message containing the
            transaction hash, transaction signature, and expiration of the
            approval.
        :param tx_params: transaction parameters
        :returns: EIP712 hash of the Coordinator approval message with the
            domain separator of this contract.
        """
        (approval) = self.validate_and_normalize_inputs(approval)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(approval).transact(tx_params.as_dict())

    def estimate_gas(
        self, approval: Tuple0x621bd3c4, tx_params: Optional[TxParams] = None
    ) -> int:
        """Estimate gas consumption of method call."""
        (approval) = self.validate_and_normalize_inputs(approval)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(approval).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self, approval: Tuple0x621bd3c4, tx_params: Optional[TxParams] = None
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (approval) = self.validate_and_normalize_inputs(approval)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(approval).buildTransaction(
            tx_params.as_dict()
        )


class ExecuteTransactionMethod(ContractMethod):
    """Various interfaces to the executeTransaction method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
    ):
        """Validate the inputs to the executeTransaction method."""
        self.validator.assert_valid(
            method_name="executeTransaction",
            parameter_name="transaction",
            argument_value=transaction,
        )
        self.validator.assert_valid(
            method_name="executeTransaction",
            parameter_name="txOrigin",
            argument_value=tx_origin,
        )
        tx_origin = self.validate_and_checksum_address(tx_origin)
        self.validator.assert_valid(
            method_name="executeTransaction",
            parameter_name="transactionSignature",
            argument_value=transaction_signature,
        )
        transaction_signature = bytes.fromhex(
            transaction_signature.decode("utf-8")
        )
        self.validator.assert_valid(
            method_name="executeTransaction",
            parameter_name="approvalExpirationTimeSeconds",
            argument_value=approval_expiration_time_seconds,
        )
        self.validator.assert_valid(
            method_name="executeTransaction",
            parameter_name="approvalSignatures",
            argument_value=approval_signatures,
        )
        approval_signatures = [
            bytes.fromhex(approval_signatures_element.decode("utf-8"))
            for approval_signatures_element in approval_signatures
        ]
        return (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )

    def call(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> Union[None, Union[HexBytes, bytes]]:
        """Execute underlying contract method via eth_call.

        Executes a 0x transaction that has been signed by the feeRecipients
        that correspond to each order in the transaction's Exchange calldata.

        :param approvalExpirationTimeSeconds: Array of expiration times in
            seconds for which each corresponding approval signature expires.
        :param approvalSignatures: Array of signatures that correspond to the
            feeRecipients of each order in the transaction's Exchange calldata.
        :param transaction: 0x transaction containing salt, signerAddress, and
            data.
        :param transactionSignature: Proof that the transaction has been signed
            by the signer.
        :param txOrigin: Required signer of Ethereum transaction calling this
            function.
        :param tx_params: transaction parameters
        :returns: the return value of the underlying method.
        """
        (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ) = self.validate_and_normalize_inputs(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ).call(tx_params.as_dict())

    def send_transaction(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Executes a 0x transaction that has been signed by the feeRecipients
        that correspond to each order in the transaction's Exchange calldata.

        :param approvalExpirationTimeSeconds: Array of expiration times in
            seconds for which each corresponding approval signature expires.
        :param approvalSignatures: Array of signatures that correspond to the
            feeRecipients of each order in the transaction's Exchange calldata.
        :param transaction: 0x transaction containing salt, signerAddress, and
            data.
        :param transactionSignature: Proof that the transaction has been signed
            by the signer.
        :param txOrigin: Required signer of Ethereum transaction calling this
            function.
        :param tx_params: transaction parameters
        """
        (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ) = self.validate_and_normalize_inputs(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ).transact(tx_params.as_dict())

    def estimate_gas(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ) = self.validate_and_normalize_inputs(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ).estimateGas(tx_params.as_dict())

    def build_transaction(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ) = self.validate_and_normalize_inputs(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ).buildTransaction(tx_params.as_dict())


class Eip712ExchangeDomainHashMethod(ContractMethod):
    """Various interfaces to the EIP712_EXCHANGE_DOMAIN_HASH method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def call(self, tx_params: Optional[TxParams] = None) -> bytes:
        """Execute underlying contract method via eth_call.

        :param tx_params: transaction parameters

        """
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method().call(tx_params.as_dict())

    def send_transaction(
        self, tx_params: Optional[TxParams] = None
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        :param tx_params: transaction parameters

        """
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method().transact(tx_params.as_dict())

    def estimate_gas(self, tx_params: Optional[TxParams] = None) -> int:
        """Estimate gas consumption of method call."""
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method().estimateGas(tx_params.as_dict())

    def build_transaction(self, tx_params: Optional[TxParams] = None) -> dict:
        """Construct calldata to be used as input to the method."""
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method().buildTransaction(tx_params.as_dict())


class AssertValidCoordinatorApprovalsMethod(ContractMethod):
    """Various interfaces to the assertValidCoordinatorApprovals method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
    ):
        """Validate the inputs to the assertValidCoordinatorApprovals method."""
        self.validator.assert_valid(
            method_name="assertValidCoordinatorApprovals",
            parameter_name="transaction",
            argument_value=transaction,
        )
        self.validator.assert_valid(
            method_name="assertValidCoordinatorApprovals",
            parameter_name="txOrigin",
            argument_value=tx_origin,
        )
        tx_origin = self.validate_and_checksum_address(tx_origin)
        self.validator.assert_valid(
            method_name="assertValidCoordinatorApprovals",
            parameter_name="transactionSignature",
            argument_value=transaction_signature,
        )
        transaction_signature = bytes.fromhex(
            transaction_signature.decode("utf-8")
        )
        self.validator.assert_valid(
            method_name="assertValidCoordinatorApprovals",
            parameter_name="approvalExpirationTimeSeconds",
            argument_value=approval_expiration_time_seconds,
        )
        self.validator.assert_valid(
            method_name="assertValidCoordinatorApprovals",
            parameter_name="approvalSignatures",
            argument_value=approval_signatures,
        )
        approval_signatures = [
            bytes.fromhex(approval_signatures_element.decode("utf-8"))
            for approval_signatures_element in approval_signatures
        ]
        return (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )

    def call(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> None:
        """Execute underlying contract method via eth_call.

        Validates that the 0x transaction has been approved by all of the
        feeRecipients that correspond to each order in the transaction's
        Exchange calldata.

        :param approvalExpirationTimeSeconds: Array of expiration times in
            seconds for which each corresponding approval signature expires.
        :param approvalSignatures: Array of signatures that correspond to the
            feeRecipients of each order in the transaction's Exchange calldata.
        :param transaction: 0x transaction containing salt, signerAddress, and
            data.
        :param transactionSignature: Proof that the transaction has been signed
            by the signer.
        :param txOrigin: Required signer of Ethereum transaction calling this
            function.
        :param tx_params: transaction parameters

        """
        (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ) = self.validate_and_normalize_inputs(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ).call(tx_params.as_dict())

    def send_transaction(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Validates that the 0x transaction has been approved by all of the
        feeRecipients that correspond to each order in the transaction's
        Exchange calldata.

        :param approvalExpirationTimeSeconds: Array of expiration times in
            seconds for which each corresponding approval signature expires.
        :param approvalSignatures: Array of signatures that correspond to the
            feeRecipients of each order in the transaction's Exchange calldata.
        :param transaction: 0x transaction containing salt, signerAddress, and
            data.
        :param transactionSignature: Proof that the transaction has been signed
            by the signer.
        :param txOrigin: Required signer of Ethereum transaction calling this
            function.
        :param tx_params: transaction parameters

        """
        (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ) = self.validate_and_normalize_inputs(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ).transact(tx_params.as_dict())

    def estimate_gas(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ) = self.validate_and_normalize_inputs(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ).estimateGas(tx_params.as_dict())

    def build_transaction(
        self,
        transaction: Tuple0xa1ae4ec4,
        tx_origin: str,
        transaction_signature: bytes,
        approval_expiration_time_seconds: List[int],
        approval_signatures: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ) = self.validate_and_normalize_inputs(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            transaction,
            tx_origin,
            transaction_signature,
            approval_expiration_time_seconds,
            approval_signatures,
        ).buildTransaction(tx_params.as_dict())


class DecodeOrdersFromFillDataMethod(ContractMethod):
    """Various interfaces to the decodeOrdersFromFillData method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, data: bytes):
        """Validate the inputs to the decodeOrdersFromFillData method."""
        self.validator.assert_valid(
            method_name="decodeOrdersFromFillData",
            parameter_name="data",
            argument_value=data,
        )
        data = bytes.fromhex(data.decode("utf-8"))
        return data

    def call(
        self, data: bytes, tx_params: Optional[TxParams] = None
    ) -> List[Tuple0x260219a2]:
        """Execute underlying contract method via eth_call.

        Decodes the orders from Exchange calldata representing any fill method.

        :param data: Exchange calldata representing a fill method.
        :param tx_params: transaction parameters
        :returns: The orders from the Exchange calldata.
        """
        (data) = self.validate_and_normalize_inputs(data)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(data).call(tx_params.as_dict())

    def send_transaction(
        self, data: bytes, tx_params: Optional[TxParams] = None
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Decodes the orders from Exchange calldata representing any fill method.

        :param data: Exchange calldata representing a fill method.
        :param tx_params: transaction parameters
        :returns: The orders from the Exchange calldata.
        """
        (data) = self.validate_and_normalize_inputs(data)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(data).transact(tx_params.as_dict())

    def estimate_gas(
        self, data: bytes, tx_params: Optional[TxParams] = None
    ) -> int:
        """Estimate gas consumption of method call."""
        (data) = self.validate_and_normalize_inputs(data)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(data).estimateGas(tx_params.as_dict())

    def build_transaction(
        self, data: bytes, tx_params: Optional[TxParams] = None
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (data) = self.validate_and_normalize_inputs(data)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(data).buildTransaction(
            tx_params.as_dict()
        )


class Eip712CoordinatorDomainHashMethod(ContractMethod):
    """Various interfaces to the EIP712_COORDINATOR_DOMAIN_HASH method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def call(self, tx_params: Optional[TxParams] = None) -> bytes:
        """Execute underlying contract method via eth_call.

        :param tx_params: transaction parameters

        """
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method().call(tx_params.as_dict())

    def send_transaction(
        self, tx_params: Optional[TxParams] = None
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        :param tx_params: transaction parameters

        """
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method().transact(tx_params.as_dict())

    def estimate_gas(self, tx_params: Optional[TxParams] = None) -> int:
        """Estimate gas consumption of method call."""
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method().estimateGas(tx_params.as_dict())

    def build_transaction(self, tx_params: Optional[TxParams] = None) -> dict:
        """Construct calldata to be used as input to the method."""
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method().buildTransaction(tx_params.as_dict())


# pylint: disable=too-many-public-methods,too-many-instance-attributes
class Coordinator:
    """Wrapper class for Coordinator Solidity contract.

    All method parameters of type `bytes`:code: should be encoded as UTF-8,
    which can be accomplished via `str.encode("utf_8")`:code:.
    """

    get_signer_address: GetSignerAddressMethod
    """Constructor-initialized instance of
    :class:`GetSignerAddressMethod`.
    """

    get_transaction_hash: GetTransactionHashMethod
    """Constructor-initialized instance of
    :class:`GetTransactionHashMethod`.
    """

    get_coordinator_approval_hash: GetCoordinatorApprovalHashMethod
    """Constructor-initialized instance of
    :class:`GetCoordinatorApprovalHashMethod`.
    """

    execute_transaction: ExecuteTransactionMethod
    """Constructor-initialized instance of
    :class:`ExecuteTransactionMethod`.
    """

    eip712_exchange_domain_hash: Eip712ExchangeDomainHashMethod
    """Constructor-initialized instance of
    :class:`Eip712ExchangeDomainHashMethod`.
    """

    assert_valid_coordinator_approvals: AssertValidCoordinatorApprovalsMethod
    """Constructor-initialized instance of
    :class:`AssertValidCoordinatorApprovalsMethod`.
    """

    decode_orders_from_fill_data: DecodeOrdersFromFillDataMethod
    """Constructor-initialized instance of
    :class:`DecodeOrdersFromFillDataMethod`.
    """

    eip712_coordinator_domain_hash: Eip712CoordinatorDomainHashMethod
    """Constructor-initialized instance of
    :class:`Eip712CoordinatorDomainHashMethod`.
    """

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        validator: CoordinatorValidator = None,
    ):
        """Get an instance of wrapper for smart contract.

        :param provider: instance of :class:`web3.providers.base.BaseProvider`
        :param contract_address: where the contract has been deployed
        :param validator: for validation of method inputs.
        """
        self.contract_address = contract_address

        if not validator:
            validator = CoordinatorValidator(provider, contract_address)

        self._web3_eth = Web3(  # type: ignore # pylint: disable=no-member
            provider
        ).eth

        functions = self._web3_eth.contract(
            address=to_checksum_address(contract_address),
            abi=Coordinator.abi(),
        ).functions

        self.get_signer_address = GetSignerAddressMethod(
            provider, contract_address, functions.getSignerAddress, validator
        )

        self.get_transaction_hash = GetTransactionHashMethod(
            provider, contract_address, functions.getTransactionHash, validator
        )

        self.get_coordinator_approval_hash = GetCoordinatorApprovalHashMethod(
            provider,
            contract_address,
            functions.getCoordinatorApprovalHash,
            validator,
        )

        self.execute_transaction = ExecuteTransactionMethod(
            provider, contract_address, functions.executeTransaction, validator
        )

        self.eip712_exchange_domain_hash = Eip712ExchangeDomainHashMethod(
            provider,
            contract_address,
            functions.EIP712_EXCHANGE_DOMAIN_HASH,
            validator,
        )

        self.assert_valid_coordinator_approvals = AssertValidCoordinatorApprovalsMethod(
            provider,
            contract_address,
            functions.assertValidCoordinatorApprovals,
            validator,
        )

        self.decode_orders_from_fill_data = DecodeOrdersFromFillDataMethod(
            provider,
            contract_address,
            functions.decodeOrdersFromFillData,
            validator,
        )

        self.eip712_coordinator_domain_hash = Eip712CoordinatorDomainHashMethod(
            provider,
            contract_address,
            functions.EIP712_COORDINATOR_DOMAIN_HASH,
            validator,
        )

    @staticmethod
    def abi():
        """Return the ABI to the underlying contract."""
        return json.loads(
            '[{"constant":true,"inputs":[{"name":"hash","type":"bytes32"},{"name":"signature","type":"bytes"}],"name":"getSignerAddress","outputs":[{"name":"signerAddress","type":"address"}],"payable":false,"stateMutability":"pure","type":"function"},{"constant":true,"inputs":[{"components":[{"name":"salt","type":"uint256"},{"name":"signerAddress","type":"address"},{"name":"data","type":"bytes"}],"name":"transaction","type":"tuple"}],"name":"getTransactionHash","outputs":[{"name":"transactionHash","type":"bytes32"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"components":[{"name":"txOrigin","type":"address"},{"name":"transactionHash","type":"bytes32"},{"name":"transactionSignature","type":"bytes"},{"name":"approvalExpirationTimeSeconds","type":"uint256"}],"name":"approval","type":"tuple"}],"name":"getCoordinatorApprovalHash","outputs":[{"name":"approvalHash","type":"bytes32"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"components":[{"name":"salt","type":"uint256"},{"name":"signerAddress","type":"address"},{"name":"data","type":"bytes"}],"name":"transaction","type":"tuple"},{"name":"txOrigin","type":"address"},{"name":"transactionSignature","type":"bytes"},{"name":"approvalExpirationTimeSeconds","type":"uint256[]"},{"name":"approvalSignatures","type":"bytes[]"}],"name":"executeTransaction","outputs":[],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[],"name":"EIP712_EXCHANGE_DOMAIN_HASH","outputs":[{"name":"","type":"bytes32"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"components":[{"name":"salt","type":"uint256"},{"name":"signerAddress","type":"address"},{"name":"data","type":"bytes"}],"name":"transaction","type":"tuple"},{"name":"txOrigin","type":"address"},{"name":"transactionSignature","type":"bytes"},{"name":"approvalExpirationTimeSeconds","type":"uint256[]"},{"name":"approvalSignatures","type":"bytes[]"}],"name":"assertValidCoordinatorApprovals","outputs":[],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"name":"data","type":"bytes"}],"name":"decodeOrdersFromFillData","outputs":[{"components":[{"name":"makerAddress","type":"address"},{"name":"takerAddress","type":"address"},{"name":"feeRecipientAddress","type":"address"},{"name":"senderAddress","type":"address"},{"name":"makerAssetAmount","type":"uint256"},{"name":"takerAssetAmount","type":"uint256"},{"name":"makerFee","type":"uint256"},{"name":"takerFee","type":"uint256"},{"name":"expirationTimeSeconds","type":"uint256"},{"name":"salt","type":"uint256"},{"name":"makerAssetData","type":"bytes"},{"name":"takerAssetData","type":"bytes"}],"name":"orders","type":"tuple[]"}],"payable":false,"stateMutability":"pure","type":"function"},{"constant":true,"inputs":[],"name":"EIP712_COORDINATOR_DOMAIN_HASH","outputs":[{"name":"","type":"bytes32"}],"payable":false,"stateMutability":"view","type":"function"},{"inputs":[{"name":"_exchange","type":"address"}],"payable":false,"stateMutability":"nonpayable","type":"constructor"}]'  # noqa: E501 (line-too-long)
        )


# pylint: disable=too-many-lines



"""Generated wrapper for OrderValidator Solidity contract."""

# pylint: disable=too-many-arguments

import json
from typing import (  # pylint: disable=unused-import
    Any,
    List,
    Optional,
    Tuple,
    Union,
)

from eth_utils import to_checksum_address
from mypy_extensions import TypedDict  # pylint: disable=unused-import
from hexbytes import HexBytes
from web3 import Web3
from web3.contract import ContractFunction
from web3.datastructures import AttributeDict
from web3.providers.base import BaseProvider

from zero_ex.contract_wrappers.bases import ContractMethod, Validator
from zero_ex.contract_wrappers.tx_params import TxParams


# Try to import a custom validator class definition; if there isn't one,
# declare one that we can instantiate for the default argument to the
# constructor for OrderValidator below.
try:
    # both mypy and pylint complain about what we're doing here, but this
    # works just fine, so their messages have been disabled here.
    from . import (  # type: ignore # pylint: disable=import-self
        OrderValidatorValidator,
    )
except ImportError:

    class OrderValidatorValidator(  # type: ignore
        Validator
    ):
        """No-op input validator."""


class Tuple0x260219a2(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    makerAddress: str

    takerAddress: str

    feeRecipientAddress: str

    senderAddress: str

    makerAssetAmount: int

    takerAssetAmount: int

    makerFee: int

    takerFee: int

    expirationTimeSeconds: int

    salt: int

    makerAssetData: bytes

    takerAssetData: bytes


class Tuple0xb1e4a1ae(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    orderStatus: int

    orderHash: bytes

    orderTakerAssetFilledAmount: int


class Tuple0x8cfc0927(TypedDict):
    """Python representation of a tuple or struct.

    Solidity compiler output does not include the names of structs that appear
    in method definitions.  A tuple found in an ABI may have been written in
    Solidity as a literal, anonymous tuple, or it may have been written as a
    named `struct`:code:, but there is no way to tell from the compiler
    output.  This class represents a tuple that appeared in a method
    definition.  Its name is derived from a hash of that tuple's field names,
    and every method whose ABI refers to a tuple with that same list of field
    names will have a generated wrapper method that refers to this class.

    Any members of type `bytes`:code: should be encoded as UTF-8, which can be
    accomplished via `str.encode("utf_8")`:code:
    """

    makerBalance: int

    makerAllowance: int

    takerBalance: int

    takerAllowance: int

    makerZrxBalance: int

    makerZrxAllowance: int

    takerZrxBalance: int

    takerZrxAllowance: int


class GetOrderAndTraderInfoMethod(ContractMethod):
    """Various interfaces to the getOrderAndTraderInfo method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(
        self, order: Tuple0x260219a2, taker_address: str
    ):
        """Validate the inputs to the getOrderAndTraderInfo method."""
        self.validator.assert_valid(
            method_name="getOrderAndTraderInfo",
            parameter_name="order",
            argument_value=order,
        )
        self.validator.assert_valid(
            method_name="getOrderAndTraderInfo",
            parameter_name="takerAddress",
            argument_value=taker_address,
        )
        taker_address = self.validate_and_checksum_address(taker_address)
        return (order, taker_address)

    def call(
        self,
        order: Tuple0x260219a2,
        taker_address: str,
        tx_params: Optional[TxParams] = None,
    ) -> Tuple[Tuple0xb1e4a1ae, Tuple0x8cfc0927]:
        """Execute underlying contract method via eth_call.

        :param tx_params: transaction parameters

        """
        (order, taker_address) = self.validate_and_normalize_inputs(
            order, taker_address
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order, taker_address).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self,
        order: Tuple0x260219a2,
        taker_address: str,
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        :param tx_params: transaction parameters

        """
        (order, taker_address) = self.validate_and_normalize_inputs(
            order, taker_address
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order, taker_address).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self,
        order: Tuple0x260219a2,
        taker_address: str,
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (order, taker_address) = self.validate_and_normalize_inputs(
            order, taker_address
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order, taker_address).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self,
        order: Tuple0x260219a2,
        taker_address: str,
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (order, taker_address) = self.validate_and_normalize_inputs(
            order, taker_address
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order, taker_address).buildTransaction(
            tx_params.as_dict()
        )


class GetBalanceAndAllowanceMethod(ContractMethod):
    """Various interfaces to the getBalanceAndAllowance method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, target: str, asset_data: bytes):
        """Validate the inputs to the getBalanceAndAllowance method."""
        self.validator.assert_valid(
            method_name="getBalanceAndAllowance",
            parameter_name="target",
            argument_value=target,
        )
        target = self.validate_and_checksum_address(target)
        self.validator.assert_valid(
            method_name="getBalanceAndAllowance",
            parameter_name="assetData",
            argument_value=asset_data,
        )
        asset_data = bytes.fromhex(asset_data.decode("utf-8"))
        return (target, asset_data)

    def call(
        self,
        target: str,
        asset_data: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> Tuple[int, int]:
        """Execute underlying contract method via eth_call.

        :param tx_params: transaction parameters

        """
        (target, asset_data) = self.validate_and_normalize_inputs(
            target, asset_data
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(target, asset_data).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self,
        target: str,
        asset_data: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        :param tx_params: transaction parameters

        """
        (target, asset_data) = self.validate_and_normalize_inputs(
            target, asset_data
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(target, asset_data).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self,
        target: str,
        asset_data: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (target, asset_data) = self.validate_and_normalize_inputs(
            target, asset_data
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(target, asset_data).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self,
        target: str,
        asset_data: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (target, asset_data) = self.validate_and_normalize_inputs(
            target, asset_data
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(target, asset_data).buildTransaction(
            tx_params.as_dict()
        )


class GetOrdersAndTradersInfoMethod(ContractMethod):
    """Various interfaces to the getOrdersAndTradersInfo method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(
        self, orders: List[Tuple0x260219a2], taker_addresses: List[str]
    ):
        """Validate the inputs to the getOrdersAndTradersInfo method."""
        self.validator.assert_valid(
            method_name="getOrdersAndTradersInfo",
            parameter_name="orders",
            argument_value=orders,
        )
        self.validator.assert_valid(
            method_name="getOrdersAndTradersInfo",
            parameter_name="takerAddresses",
            argument_value=taker_addresses,
        )
        return (orders, taker_addresses)

    def call(
        self,
        orders: List[Tuple0x260219a2],
        taker_addresses: List[str],
        tx_params: Optional[TxParams] = None,
    ) -> Tuple[List[Tuple0xb1e4a1ae], List[Tuple0x8cfc0927]]:
        """Execute underlying contract method via eth_call.

        :param tx_params: transaction parameters

        """
        (orders, taker_addresses) = self.validate_and_normalize_inputs(
            orders, taker_addresses
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(orders, taker_addresses).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self,
        orders: List[Tuple0x260219a2],
        taker_addresses: List[str],
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        :param tx_params: transaction parameters

        """
        (orders, taker_addresses) = self.validate_and_normalize_inputs(
            orders, taker_addresses
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(orders, taker_addresses).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self,
        orders: List[Tuple0x260219a2],
        taker_addresses: List[str],
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (orders, taker_addresses) = self.validate_and_normalize_inputs(
            orders, taker_addresses
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(orders, taker_addresses).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self,
        orders: List[Tuple0x260219a2],
        taker_addresses: List[str],
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (orders, taker_addresses) = self.validate_and_normalize_inputs(
            orders, taker_addresses
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            orders, taker_addresses
        ).buildTransaction(tx_params.as_dict())


class GetTradersInfoMethod(ContractMethod):
    """Various interfaces to the getTradersInfo method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(
        self, orders: List[Tuple0x260219a2], taker_addresses: List[str]
    ):
        """Validate the inputs to the getTradersInfo method."""
        self.validator.assert_valid(
            method_name="getTradersInfo",
            parameter_name="orders",
            argument_value=orders,
        )
        self.validator.assert_valid(
            method_name="getTradersInfo",
            parameter_name="takerAddresses",
            argument_value=taker_addresses,
        )
        return (orders, taker_addresses)

    def call(
        self,
        orders: List[Tuple0x260219a2],
        taker_addresses: List[str],
        tx_params: Optional[TxParams] = None,
    ) -> List[Tuple0x8cfc0927]:
        """Execute underlying contract method via eth_call.

        :param tx_params: transaction parameters

        """
        (orders, taker_addresses) = self.validate_and_normalize_inputs(
            orders, taker_addresses
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(orders, taker_addresses).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self,
        orders: List[Tuple0x260219a2],
        taker_addresses: List[str],
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        :param tx_params: transaction parameters

        """
        (orders, taker_addresses) = self.validate_and_normalize_inputs(
            orders, taker_addresses
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(orders, taker_addresses).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self,
        orders: List[Tuple0x260219a2],
        taker_addresses: List[str],
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (orders, taker_addresses) = self.validate_and_normalize_inputs(
            orders, taker_addresses
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(orders, taker_addresses).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self,
        orders: List[Tuple0x260219a2],
        taker_addresses: List[str],
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (orders, taker_addresses) = self.validate_and_normalize_inputs(
            orders, taker_addresses
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            orders, taker_addresses
        ).buildTransaction(tx_params.as_dict())


class GetErc721TokenOwnerMethod(ContractMethod):
    """Various interfaces to the getERC721TokenOwner method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, token: str, token_id: int):
        """Validate the inputs to the getERC721TokenOwner method."""
        self.validator.assert_valid(
            method_name="getERC721TokenOwner",
            parameter_name="token",
            argument_value=token,
        )
        token = self.validate_and_checksum_address(token)
        self.validator.assert_valid(
            method_name="getERC721TokenOwner",
            parameter_name="tokenId",
            argument_value=token_id,
        )
        # safeguard against fractional inputs
        token_id = int(token_id)
        return (token, token_id)

    def call(
        self, token: str, token_id: int, tx_params: Optional[TxParams] = None
    ) -> str:
        """Execute underlying contract method via eth_call.

        :param tx_params: transaction parameters

        """
        (token, token_id) = self.validate_and_normalize_inputs(token, token_id)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(token, token_id).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self, token: str, token_id: int, tx_params: Optional[TxParams] = None
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        :param tx_params: transaction parameters

        """
        (token, token_id) = self.validate_and_normalize_inputs(token, token_id)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(token, token_id).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self, token: str, token_id: int, tx_params: Optional[TxParams] = None
    ) -> int:
        """Estimate gas consumption of method call."""
        (token, token_id) = self.validate_and_normalize_inputs(token, token_id)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(token, token_id).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self, token: str, token_id: int, tx_params: Optional[TxParams] = None
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (token, token_id) = self.validate_and_normalize_inputs(token, token_id)
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(token, token_id).buildTransaction(
            tx_params.as_dict()
        )


class GetBalancesAndAllowancesMethod(ContractMethod):
    """Various interfaces to the getBalancesAndAllowances method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(
        self, target: str, asset_data: List[bytes]
    ):
        """Validate the inputs to the getBalancesAndAllowances method."""
        self.validator.assert_valid(
            method_name="getBalancesAndAllowances",
            parameter_name="target",
            argument_value=target,
        )
        target = self.validate_and_checksum_address(target)
        self.validator.assert_valid(
            method_name="getBalancesAndAllowances",
            parameter_name="assetData",
            argument_value=asset_data,
        )
        asset_data = [
            bytes.fromhex(asset_data_element.decode("utf-8"))
            for asset_data_element in asset_data
        ]
        return (target, asset_data)

    def call(
        self,
        target: str,
        asset_data: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> Tuple[List[int], List[int]]:
        """Execute underlying contract method via eth_call.

        :param tx_params: transaction parameters

        """
        (target, asset_data) = self.validate_and_normalize_inputs(
            target, asset_data
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(target, asset_data).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self,
        target: str,
        asset_data: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        :param tx_params: transaction parameters

        """
        (target, asset_data) = self.validate_and_normalize_inputs(
            target, asset_data
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(target, asset_data).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self,
        target: str,
        asset_data: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (target, asset_data) = self.validate_and_normalize_inputs(
            target, asset_data
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(target, asset_data).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self,
        target: str,
        asset_data: List[bytes],
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (target, asset_data) = self.validate_and_normalize_inputs(
            target, asset_data
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(target, asset_data).buildTransaction(
            tx_params.as_dict()
        )


class GetTraderInfoMethod(ContractMethod):
    """Various interfaces to the getTraderInfo method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(
        self, order: Tuple0x260219a2, taker_address: str
    ):
        """Validate the inputs to the getTraderInfo method."""
        self.validator.assert_valid(
            method_name="getTraderInfo",
            parameter_name="order",
            argument_value=order,
        )
        self.validator.assert_valid(
            method_name="getTraderInfo",
            parameter_name="takerAddress",
            argument_value=taker_address,
        )
        taker_address = self.validate_and_checksum_address(taker_address)
        return (order, taker_address)

    def call(
        self,
        order: Tuple0x260219a2,
        taker_address: str,
        tx_params: Optional[TxParams] = None,
    ) -> Tuple0x8cfc0927:
        """Execute underlying contract method via eth_call.

        :param tx_params: transaction parameters

        """
        (order, taker_address) = self.validate_and_normalize_inputs(
            order, taker_address
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order, taker_address).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self,
        order: Tuple0x260219a2,
        taker_address: str,
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        :param tx_params: transaction parameters

        """
        (order, taker_address) = self.validate_and_normalize_inputs(
            order, taker_address
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order, taker_address).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self,
        order: Tuple0x260219a2,
        taker_address: str,
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (order, taker_address) = self.validate_and_normalize_inputs(
            order, taker_address
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order, taker_address).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self,
        order: Tuple0x260219a2,
        taker_address: str,
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (order, taker_address) = self.validate_and_normalize_inputs(
            order, taker_address
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(order, taker_address).buildTransaction(
            tx_params.as_dict()
        )


# pylint: disable=too-many-public-methods,too-many-instance-attributes
class OrderValidator:
    """Wrapper class for OrderValidator Solidity contract.

    All method parameters of type `bytes`:code: should be encoded as UTF-8,
    which can be accomplished via `str.encode("utf_8")`:code:.
    """

    get_order_and_trader_info: GetOrderAndTraderInfoMethod
    """Constructor-initialized instance of
    :class:`GetOrderAndTraderInfoMethod`.
    """

    get_balance_and_allowance: GetBalanceAndAllowanceMethod
    """Constructor-initialized instance of
    :class:`GetBalanceAndAllowanceMethod`.
    """

    get_orders_and_traders_info: GetOrdersAndTradersInfoMethod
    """Constructor-initialized instance of
    :class:`GetOrdersAndTradersInfoMethod`.
    """

    get_traders_info: GetTradersInfoMethod
    """Constructor-initialized instance of
    :class:`GetTradersInfoMethod`.
    """

    get_erc721_token_owner: GetErc721TokenOwnerMethod
    """Constructor-initialized instance of
    :class:`GetErc721TokenOwnerMethod`.
    """

    get_balances_and_allowances: GetBalancesAndAllowancesMethod
    """Constructor-initialized instance of
    :class:`GetBalancesAndAllowancesMethod`.
    """

    get_trader_info: GetTraderInfoMethod
    """Constructor-initialized instance of
    :class:`GetTraderInfoMethod`.
    """

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        validator: OrderValidatorValidator = None,
    ):
        """Get an instance of wrapper for smart contract.

        :param provider: instance of :class:`web3.providers.base.BaseProvider`
        :param contract_address: where the contract has been deployed
        :param validator: for validation of method inputs.
        """
        self.contract_address = contract_address

        if not validator:
            validator = OrderValidatorValidator(provider, contract_address)

        self._web3_eth = Web3(  # type: ignore # pylint: disable=no-member
            provider
        ).eth

        functions = self._web3_eth.contract(
            address=to_checksum_address(contract_address),
            abi=OrderValidator.abi(),
        ).functions

        self.get_order_and_trader_info = GetOrderAndTraderInfoMethod(
            provider,
            contract_address,
            functions.getOrderAndTraderInfo,
            validator,
        )

        self.get_balance_and_allowance = GetBalanceAndAllowanceMethod(
            provider,
            contract_address,
            functions.getBalanceAndAllowance,
            validator,
        )

        self.get_orders_and_traders_info = GetOrdersAndTradersInfoMethod(
            provider,
            contract_address,
            functions.getOrdersAndTradersInfo,
            validator,
        )

        self.get_traders_info = GetTradersInfoMethod(
            provider, contract_address, functions.getTradersInfo, validator
        )

        self.get_erc721_token_owner = GetErc721TokenOwnerMethod(
            provider,
            contract_address,
            functions.getERC721TokenOwner,
            validator,
        )

        self.get_balances_and_allowances = GetBalancesAndAllowancesMethod(
            provider,
            contract_address,
            functions.getBalancesAndAllowances,
            validator,
        )

        self.get_trader_info = GetTraderInfoMethod(
            provider, contract_address, functions.getTraderInfo, validator
        )

    @staticmethod
    def abi():
        """Return the ABI to the underlying contract."""
        return json.loads(
            '[{"constant":true,"inputs":[{"components":[{"name":"makerAddress","type":"address"},{"name":"takerAddress","type":"address"},{"name":"feeRecipientAddress","type":"address"},{"name":"senderAddress","type":"address"},{"name":"makerAssetAmount","type":"uint256"},{"name":"takerAssetAmount","type":"uint256"},{"name":"makerFee","type":"uint256"},{"name":"takerFee","type":"uint256"},{"name":"expirationTimeSeconds","type":"uint256"},{"name":"salt","type":"uint256"},{"name":"makerAssetData","type":"bytes"},{"name":"takerAssetData","type":"bytes"}],"name":"order","type":"tuple"},{"name":"takerAddress","type":"address"}],"name":"getOrderAndTraderInfo","outputs":[{"components":[{"name":"orderStatus","type":"uint8"},{"name":"orderHash","type":"bytes32"},{"name":"orderTakerAssetFilledAmount","type":"uint256"}],"name":"orderInfo","type":"tuple"},{"components":[{"name":"makerBalance","type":"uint256"},{"name":"makerAllowance","type":"uint256"},{"name":"takerBalance","type":"uint256"},{"name":"takerAllowance","type":"uint256"},{"name":"makerZrxBalance","type":"uint256"},{"name":"makerZrxAllowance","type":"uint256"},{"name":"takerZrxBalance","type":"uint256"},{"name":"takerZrxAllowance","type":"uint256"}],"name":"traderInfo","type":"tuple"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"name":"target","type":"address"},{"name":"assetData","type":"bytes"}],"name":"getBalanceAndAllowance","outputs":[{"name":"balance","type":"uint256"},{"name":"allowance","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"components":[{"name":"makerAddress","type":"address"},{"name":"takerAddress","type":"address"},{"name":"feeRecipientAddress","type":"address"},{"name":"senderAddress","type":"address"},{"name":"makerAssetAmount","type":"uint256"},{"name":"takerAssetAmount","type":"uint256"},{"name":"makerFee","type":"uint256"},{"name":"takerFee","type":"uint256"},{"name":"expirationTimeSeconds","type":"uint256"},{"name":"salt","type":"uint256"},{"name":"makerAssetData","type":"bytes"},{"name":"takerAssetData","type":"bytes"}],"name":"orders","type":"tuple[]"},{"name":"takerAddresses","type":"address[]"}],"name":"getOrdersAndTradersInfo","outputs":[{"components":[{"name":"orderStatus","type":"uint8"},{"name":"orderHash","type":"bytes32"},{"name":"orderTakerAssetFilledAmount","type":"uint256"}],"name":"ordersInfo","type":"tuple[]"},{"components":[{"name":"makerBalance","type":"uint256"},{"name":"makerAllowance","type":"uint256"},{"name":"takerBalance","type":"uint256"},{"name":"takerAllowance","type":"uint256"},{"name":"makerZrxBalance","type":"uint256"},{"name":"makerZrxAllowance","type":"uint256"},{"name":"takerZrxBalance","type":"uint256"},{"name":"takerZrxAllowance","type":"uint256"}],"name":"tradersInfo","type":"tuple[]"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"components":[{"name":"makerAddress","type":"address"},{"name":"takerAddress","type":"address"},{"name":"feeRecipientAddress","type":"address"},{"name":"senderAddress","type":"address"},{"name":"makerAssetAmount","type":"uint256"},{"name":"takerAssetAmount","type":"uint256"},{"name":"makerFee","type":"uint256"},{"name":"takerFee","type":"uint256"},{"name":"expirationTimeSeconds","type":"uint256"},{"name":"salt","type":"uint256"},{"name":"makerAssetData","type":"bytes"},{"name":"takerAssetData","type":"bytes"}],"name":"orders","type":"tuple[]"},{"name":"takerAddresses","type":"address[]"}],"name":"getTradersInfo","outputs":[{"components":[{"name":"makerBalance","type":"uint256"},{"name":"makerAllowance","type":"uint256"},{"name":"takerBalance","type":"uint256"},{"name":"takerAllowance","type":"uint256"},{"name":"makerZrxBalance","type":"uint256"},{"name":"makerZrxAllowance","type":"uint256"},{"name":"takerZrxBalance","type":"uint256"},{"name":"takerZrxAllowance","type":"uint256"}],"name":"","type":"tuple[]"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"name":"token","type":"address"},{"name":"tokenId","type":"uint256"}],"name":"getERC721TokenOwner","outputs":[{"name":"owner","type":"address"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"name":"target","type":"address"},{"name":"assetData","type":"bytes[]"}],"name":"getBalancesAndAllowances","outputs":[{"name":"","type":"uint256[]"},{"name":"","type":"uint256[]"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"components":[{"name":"makerAddress","type":"address"},{"name":"takerAddress","type":"address"},{"name":"feeRecipientAddress","type":"address"},{"name":"senderAddress","type":"address"},{"name":"makerAssetAmount","type":"uint256"},{"name":"takerAssetAmount","type":"uint256"},{"name":"makerFee","type":"uint256"},{"name":"takerFee","type":"uint256"},{"name":"expirationTimeSeconds","type":"uint256"},{"name":"salt","type":"uint256"},{"name":"makerAssetData","type":"bytes"},{"name":"takerAssetData","type":"bytes"}],"name":"order","type":"tuple"},{"name":"takerAddress","type":"address"}],"name":"getTraderInfo","outputs":[{"components":[{"name":"makerBalance","type":"uint256"},{"name":"makerAllowance","type":"uint256"},{"name":"takerBalance","type":"uint256"},{"name":"takerAllowance","type":"uint256"},{"name":"makerZrxBalance","type":"uint256"},{"name":"makerZrxAllowance","type":"uint256"},{"name":"takerZrxBalance","type":"uint256"},{"name":"takerZrxAllowance","type":"uint256"}],"name":"traderInfo","type":"tuple"}],"payable":false,"stateMutability":"view","type":"function"},{"inputs":[{"name":"_exchange","type":"address"},{"name":"_zrxAssetData","type":"bytes"}],"payable":false,"stateMutability":"nonpayable","type":"constructor"}]'  # noqa: E501 (line-too-long)
        )


# pylint: disable=too-many-lines



"""Generated wrapper for CoordinatorRegistry Solidity contract."""

# pylint: disable=too-many-arguments

import json
from typing import (  # pylint: disable=unused-import
    Any,
    List,
    Optional,
    Tuple,
    Union,
)

from eth_utils import to_checksum_address
from mypy_extensions import TypedDict  # pylint: disable=unused-import
from hexbytes import HexBytes
from web3 import Web3
from web3.contract import ContractFunction
from web3.datastructures import AttributeDict
from web3.providers.base import BaseProvider

from zero_ex.contract_wrappers.bases import ContractMethod, Validator
from zero_ex.contract_wrappers.tx_params import TxParams


# Try to import a custom validator class definition; if there isn't one,
# declare one that we can instantiate for the default argument to the
# constructor for CoordinatorRegistry below.
try:
    # both mypy and pylint complain about what we're doing here, but this
    # works just fine, so their messages have been disabled here.
    from . import (  # type: ignore # pylint: disable=import-self
        CoordinatorRegistryValidator,
    )
except ImportError:

    class CoordinatorRegistryValidator(  # type: ignore
        Validator
    ):
        """No-op input validator."""


class SetCoordinatorEndpointMethod(ContractMethod):
    """Various interfaces to the setCoordinatorEndpoint method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, coordinator_endpoint: str):
        """Validate the inputs to the setCoordinatorEndpoint method."""
        self.validator.assert_valid(
            method_name="setCoordinatorEndpoint",
            parameter_name="coordinatorEndpoint",
            argument_value=coordinator_endpoint,
        )
        return coordinator_endpoint

    def call(
        self, coordinator_endpoint: str, tx_params: Optional[TxParams] = None
    ) -> Union[None, Union[HexBytes, bytes]]:
        """Execute underlying contract method via eth_call.

        Called by a Coordinator operator to set the endpoint of their
        Coordinator.

        :param coordinatorEndpoint: endpoint of the Coordinator.
        :param tx_params: transaction parameters
        :returns: the return value of the underlying method.
        """
        (coordinator_endpoint) = self.validate_and_normalize_inputs(
            coordinator_endpoint
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(coordinator_endpoint).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self, coordinator_endpoint: str, tx_params: Optional[TxParams] = None
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Called by a Coordinator operator to set the endpoint of their
        Coordinator.

        :param coordinatorEndpoint: endpoint of the Coordinator.
        :param tx_params: transaction parameters
        """
        (coordinator_endpoint) = self.validate_and_normalize_inputs(
            coordinator_endpoint
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(coordinator_endpoint).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self, coordinator_endpoint: str, tx_params: Optional[TxParams] = None
    ) -> int:
        """Estimate gas consumption of method call."""
        (coordinator_endpoint) = self.validate_and_normalize_inputs(
            coordinator_endpoint
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(coordinator_endpoint).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self, coordinator_endpoint: str, tx_params: Optional[TxParams] = None
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (coordinator_endpoint) = self.validate_and_normalize_inputs(
            coordinator_endpoint
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(coordinator_endpoint).buildTransaction(
            tx_params.as_dict()
        )


class GetCoordinatorEndpointMethod(ContractMethod):
    """Various interfaces to the getCoordinatorEndpoint method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(self, coordinator_operator: str):
        """Validate the inputs to the getCoordinatorEndpoint method."""
        self.validator.assert_valid(
            method_name="getCoordinatorEndpoint",
            parameter_name="coordinatorOperator",
            argument_value=coordinator_operator,
        )
        coordinator_operator = self.validate_and_checksum_address(
            coordinator_operator
        )
        return coordinator_operator

    def call(
        self, coordinator_operator: str, tx_params: Optional[TxParams] = None
    ) -> str:
        """Execute underlying contract method via eth_call.

        Gets the endpoint for a Coordinator.

        :param coordinatorOperator: operator of the Coordinator endpoint.
        :param tx_params: transaction parameters

        """
        (coordinator_operator) = self.validate_and_normalize_inputs(
            coordinator_operator
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(coordinator_operator).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self, coordinator_operator: str, tx_params: Optional[TxParams] = None
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Gets the endpoint for a Coordinator.

        :param coordinatorOperator: operator of the Coordinator endpoint.
        :param tx_params: transaction parameters

        """
        (coordinator_operator) = self.validate_and_normalize_inputs(
            coordinator_operator
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(coordinator_operator).transact(
            tx_params.as_dict()
        )

    def estimate_gas(
        self, coordinator_operator: str, tx_params: Optional[TxParams] = None
    ) -> int:
        """Estimate gas consumption of method call."""
        (coordinator_operator) = self.validate_and_normalize_inputs(
            coordinator_operator
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(coordinator_operator).estimateGas(
            tx_params.as_dict()
        )

    def build_transaction(
        self, coordinator_operator: str, tx_params: Optional[TxParams] = None
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (coordinator_operator) = self.validate_and_normalize_inputs(
            coordinator_operator
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(coordinator_operator).buildTransaction(
            tx_params.as_dict()
        )


# pylint: disable=too-many-public-methods,too-many-instance-attributes
class CoordinatorRegistry:
    """Wrapper class for CoordinatorRegistry Solidity contract."""

    set_coordinator_endpoint: SetCoordinatorEndpointMethod
    """Constructor-initialized instance of
    :class:`SetCoordinatorEndpointMethod`.
    """

    get_coordinator_endpoint: GetCoordinatorEndpointMethod
    """Constructor-initialized instance of
    :class:`GetCoordinatorEndpointMethod`.
    """

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        validator: CoordinatorRegistryValidator = None,
    ):
        """Get an instance of wrapper for smart contract.

        :param provider: instance of :class:`web3.providers.base.BaseProvider`
        :param contract_address: where the contract has been deployed
        :param validator: for validation of method inputs.
        """
        self.contract_address = contract_address

        if not validator:
            validator = CoordinatorRegistryValidator(
                provider, contract_address
            )

        self._web3_eth = Web3(  # type: ignore # pylint: disable=no-member
            provider
        ).eth

        functions = self._web3_eth.contract(
            address=to_checksum_address(contract_address),
            abi=CoordinatorRegistry.abi(),
        ).functions

        self.set_coordinator_endpoint = SetCoordinatorEndpointMethod(
            provider,
            contract_address,
            functions.setCoordinatorEndpoint,
            validator,
        )

        self.get_coordinator_endpoint = GetCoordinatorEndpointMethod(
            provider,
            contract_address,
            functions.getCoordinatorEndpoint,
            validator,
        )

    def get_coordinator_endpoint_set_event(
        self, tx_hash: Union[HexBytes, bytes]
    ) -> Tuple[AttributeDict]:
        """Get log entry for CoordinatorEndpointSet event.

        :param tx_hash: hash of transaction emitting CoordinatorEndpointSet
            event
        """
        tx_receipt = self._web3_eth.getTransactionReceipt(tx_hash)
        return (
            self._web3_eth.contract(
                address=to_checksum_address(self.contract_address),
                abi=CoordinatorRegistry.abi(),
            )
            .events.CoordinatorEndpointSet()
            .processReceipt(tx_receipt)
        )

    @staticmethod
    def abi():
        """Return the ABI to the underlying contract."""
        return json.loads(
            '[{"constant":false,"inputs":[{"name":"coordinatorEndpoint","type":"string"}],"name":"setCoordinatorEndpoint","outputs":[],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[{"name":"coordinatorOperator","type":"address"}],"name":"getCoordinatorEndpoint","outputs":[{"name":"coordinatorEndpoint","type":"string"}],"payable":false,"stateMutability":"view","type":"function"},{"inputs":[],"payable":false,"stateMutability":"nonpayable","type":"constructor"},{"anonymous":false,"inputs":[{"indexed":false,"name":"coordinatorOperator","type":"address"},{"indexed":false,"name":"coordinatorEndpoint","type":"string"}],"name":"CoordinatorEndpointSet","type":"event"}]'  # noqa: E501 (line-too-long)
        )


# pylint: disable=too-many-lines



"""Conveniences for handling types representing Exchange Solidity structs.

The `TypedDict`:code: classes in the .exchange module represent tuples
encountered in the Exchange contract's ABI.  However, they have weird names,
containing hashes of the tuple's field names, because the name of a Solidity
`struct`:code: isn't conveyed through the ABI.  This module provides type
aliases with human-friendly names.

Converting between the JSON wire format and the types accepted by Web3.py (eg
`bytes` vs `str`) can be onerous.  This module provides conveniences for
converting Exchange structs between JSON and Python objects.
"""

from copy import copy
from typing import cast, Dict

from eth_utils import remove_0x_prefix

from zero_ex.json_schemas import assert_valid

from . import (
    Tuple0xbb41e5b3,
    Tuple0x260219a2,
    Tuple0x054ca44e,
    Tuple0xb1e4a1ae,
)


# Would rather not repeat ourselves below, but base classes are mentioned in
# the class docstrings because of a bug in sphinx rendering.  Using the `..
# autoclass` directive, with the `:show-inheritance:` role, results in docs
# being rendered with just "Bases: dict", and no mention of the direct ancestor
# of each of these classes.


class FillResults(Tuple0xbb41e5b3):
    """The `FillResults`:code: Solidity struct.

    Also known as
    `zero_ex.contract_wrappers.exchange.Tuple0xbb41e5b3`:py:class:.
    """


class Order(Tuple0x260219a2):
    """The `Order`:code: Solidity struct.

    Also known as
    `zero_ex.contract_wrappers.exchange.Tuple0x260219a2`:py:class:.
    """


class MatchedFillResults(Tuple0x054ca44e):
    """The `MatchedFillResults`:code: Solidity struct.

    Also known as
    `zero_ex.contract_wrappers.exchange.Tuple0x054ca44e`:py:class:.
    """


class OrderInfo(Tuple0xb1e4a1ae):
    """The `OrderInfo`:code: Solidity struct.

    Also known as
    `zero_ex.contract_wrappers.exchange.Tuple0xb1e4a1ae`:py:class:.
    """


def order_to_jsdict(
    order: Order,
    exchange_address="0x0000000000000000000000000000000000000000",
    signature: str = None,
) -> dict:
    """Convert a Web3-compatible order struct to a JSON-schema-compatible dict.

    More specifically, do explicit decoding for the `bytes`:code: fields, and
    convert numerics to strings.

    >>> import pprint
    >>> pprint.pprint(order_to_jsdict(
    ...     {
    ...         'makerAddress': "0x0000000000000000000000000000000000000000",
    ...         'takerAddress': "0x0000000000000000000000000000000000000000",
    ...         'feeRecipientAddress':
    ...             "0x0000000000000000000000000000000000000000",
    ...         'senderAddress': "0x0000000000000000000000000000000000000000",
    ...         'makerAssetAmount': 1,
    ...         'takerAssetAmount': 1,
    ...         'makerFee': 0,
    ...         'takerFee': 0,
    ...         'expirationTimeSeconds': 1,
    ...         'salt': 1,
    ...         'makerAssetData': (0).to_bytes(1, byteorder='big') * 20,
    ...         'takerAssetData': (0).to_bytes(1, byteorder='big') * 20,
    ...     },
    ... ))
    {'exchangeAddress': '0x0000000000000000000000000000000000000000',
     'expirationTimeSeconds': '1',
     'feeRecipientAddress': '0x0000000000000000000000000000000000000000',
     'makerAddress': '0x0000000000000000000000000000000000000000',
     'makerAssetAmount': '1',
     'makerAssetData': '0x0000000000000000000000000000000000000000',
     'makerFee': '0',
     'salt': '1',
     'senderAddress': '0x0000000000000000000000000000000000000000',
     'takerAddress': '0x0000000000000000000000000000000000000000',
     'takerAssetAmount': '1',
     'takerAssetData': '0x0000000000000000000000000000000000000000',
     'takerFee': '0'}
    """
    jsdict = cast(Dict, copy(order))

    # encode bytes fields
    jsdict["makerAssetData"] = "0x" + order["makerAssetData"].hex()
    jsdict["takerAssetData"] = "0x" + order["takerAssetData"].hex()

    jsdict["exchangeAddress"] = exchange_address

    jsdict["expirationTimeSeconds"] = str(order["expirationTimeSeconds"])

    jsdict["makerAssetAmount"] = str(order["makerAssetAmount"])
    jsdict["takerAssetAmount"] = str(order["takerAssetAmount"])

    jsdict["makerFee"] = str(order["makerFee"])
    jsdict["takerFee"] = str(order["takerFee"])

    jsdict["salt"] = str(order["salt"])

    if signature is not None:
        jsdict["signature"] = signature

    assert_valid(jsdict, "/orderSchema")

    return jsdict


def jsdict_to_order(jsdict: dict) -> Order:
    r"""Convert a JSON-schema-compatible dict order to a Web3-compatible struct.

    More specifically, do explicit encoding of the `bytes`:code: fields, and
    parse integers from strings.

    >>> import pprint
    >>> pprint.pprint(jsdict_to_order(
    ...     {
    ...         'makerAddress': "0x0000000000000000000000000000000000000000",
    ...         'takerAddress': "0x0000000000000000000000000000000000000000",
    ...         'feeRecipientAddress': "0x0000000000000000000000000000000000000000",
    ...         'senderAddress': "0x0000000000000000000000000000000000000000",
    ...         'makerAssetAmount': "1000000000000000000",
    ...         'takerAssetAmount': "1000000000000000000",
    ...         'makerFee': "0",
    ...         'takerFee': "0",
    ...         'expirationTimeSeconds': "12345",
    ...         'salt': "12345",
    ...         'makerAssetData': "0x0000000000000000000000000000000000000000",
    ...         'takerAssetData': "0x0000000000000000000000000000000000000000",
    ...         'exchangeAddress': "0x0000000000000000000000000000000000000000",
    ...     },
    ... ))
    {'expirationTimeSeconds': 12345,
     'feeRecipientAddress': '0x0000000000000000000000000000000000000000',
     'makerAddress': '0x0000000000000000000000000000000000000000',
     'makerAssetAmount': 1000000000000000000,
     'makerAssetData': b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
                       b'\x00\x00\x00\x00\x00\x00\x00\x00',
     'makerFee': 0,
     'salt': 12345,
     'senderAddress': '0x0000000000000000000000000000000000000000',
     'takerAddress': '0x0000000000000000000000000000000000000000',
     'takerAssetAmount': 1000000000000000000,
     'takerAssetData': b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
                       b'\x00\x00\x00\x00\x00\x00\x00\x00',
     'takerFee': 0}
    """  # noqa: E501 (line too long)
    assert_valid(jsdict, "/orderSchema")

    order = cast(Order, copy(jsdict))

    order["makerAssetData"] = bytes.fromhex(
        remove_0x_prefix(jsdict["makerAssetData"])
    )
    order["takerAssetData"] = bytes.fromhex(
        remove_0x_prefix(jsdict["takerAssetData"])
    )

    order["makerAssetAmount"] = int(jsdict["makerAssetAmount"])
    order["takerAssetAmount"] = int(jsdict["takerAssetAmount"])

    order["makerFee"] = int(jsdict["makerFee"])
    order["takerFee"] = int(jsdict["takerFee"])

    order["expirationTimeSeconds"] = int(jsdict["expirationTimeSeconds"])

    order["salt"] = int(jsdict["salt"])

    del order["exchangeAddress"]  # type: ignore
    # silence mypy pending release of
    # https://github.com/python/mypy/issues/3550

    return order



"""Generated wrapper for IValidator Solidity contract."""

# pylint: disable=too-many-arguments

import json
from typing import (  # pylint: disable=unused-import
    Any,
    List,
    Optional,
    Tuple,
    Union,
)

from eth_utils import to_checksum_address
from mypy_extensions import TypedDict  # pylint: disable=unused-import
from hexbytes import HexBytes
from web3 import Web3
from web3.contract import ContractFunction
from web3.datastructures import AttributeDict
from web3.providers.base import BaseProvider

from zero_ex.contract_wrappers.bases import ContractMethod, Validator
from zero_ex.contract_wrappers.tx_params import TxParams


# Try to import a custom validator class definition; if there isn't one,
# declare one that we can instantiate for the default argument to the
# constructor for IValidator below.
try:
    # both mypy and pylint complain about what we're doing here, but this
    # works just fine, so their messages have been disabled here.
    from . import (  # type: ignore # pylint: disable=import-self
        IValidatorValidator,
    )
except ImportError:

    class IValidatorValidator(  # type: ignore
        Validator
    ):
        """No-op input validator."""


class IsValidSignatureMethod(ContractMethod):
    """Various interfaces to the isValidSignature method."""

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        contract_function: ContractFunction,
        validator: Validator = None,
    ):
        """Persist instance data."""
        super().__init__(provider, contract_address, validator)
        self._underlying_method = contract_function

    def validate_and_normalize_inputs(
        self, _hash: bytes, signer_address: str, signature: bytes
    ):
        """Validate the inputs to the isValidSignature method."""
        self.validator.assert_valid(
            method_name="isValidSignature",
            parameter_name="hash",
            argument_value=_hash,
        )
        self.validator.assert_valid(
            method_name="isValidSignature",
            parameter_name="signerAddress",
            argument_value=signer_address,
        )
        signer_address = self.validate_and_checksum_address(signer_address)
        self.validator.assert_valid(
            method_name="isValidSignature",
            parameter_name="signature",
            argument_value=signature,
        )
        signature = bytes.fromhex(signature.decode("utf-8"))
        return (_hash, signer_address, signature)

    def call(
        self,
        _hash: bytes,
        signer_address: str,
        signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> bytes:
        """Execute underlying contract method via eth_call.

        Verifies that a signature is valid.

        :param hash: Message hash that is signed.
        :param signature: Proof of signing.
        :param signerAddress: Address that should have signed the given hash.
        :param tx_params: transaction parameters
        :returns: Magic bytes4 value if the signature is valid. Magic value is
            bytes4(keccak256("isValidValidatorSignature(address,bytes32,address,by-
            tes)"))
        """
        (
            _hash,
            signer_address,
            signature,
        ) = self.validate_and_normalize_inputs(
            _hash, signer_address, signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(_hash, signer_address, signature).call(
            tx_params.as_dict()
        )

    def send_transaction(
        self,
        _hash: bytes,
        signer_address: str,
        signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> Union[HexBytes, bytes]:
        """Execute underlying contract method via eth_sendTransaction.

        Verifies that a signature is valid.

        :param hash: Message hash that is signed.
        :param signature: Proof of signing.
        :param signerAddress: Address that should have signed the given hash.
        :param tx_params: transaction parameters
        :returns: Magic bytes4 value if the signature is valid. Magic value is
            bytes4(keccak256("isValidValidatorSignature(address,bytes32,address,by-
            tes)"))
        """
        (
            _hash,
            signer_address,
            signature,
        ) = self.validate_and_normalize_inputs(
            _hash, signer_address, signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            _hash, signer_address, signature
        ).transact(tx_params.as_dict())

    def estimate_gas(
        self,
        _hash: bytes,
        signer_address: str,
        signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> int:
        """Estimate gas consumption of method call."""
        (
            _hash,
            signer_address,
            signature,
        ) = self.validate_and_normalize_inputs(
            _hash, signer_address, signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            _hash, signer_address, signature
        ).estimateGas(tx_params.as_dict())

    def build_transaction(
        self,
        _hash: bytes,
        signer_address: str,
        signature: bytes,
        tx_params: Optional[TxParams] = None,
    ) -> dict:
        """Construct calldata to be used as input to the method."""
        (
            _hash,
            signer_address,
            signature,
        ) = self.validate_and_normalize_inputs(
            _hash, signer_address, signature
        )
        tx_params = super().normalize_tx_params(tx_params)
        return self._underlying_method(
            _hash, signer_address, signature
        ).buildTransaction(tx_params.as_dict())


# pylint: disable=too-many-public-methods,too-many-instance-attributes
class IValidator:
    """Wrapper class for IValidator Solidity contract.

    All method parameters of type `bytes`:code: should be encoded as UTF-8,
    which can be accomplished via `str.encode("utf_8")`:code:.
    """

    is_valid_signature: IsValidSignatureMethod
    """Constructor-initialized instance of
    :class:`IsValidSignatureMethod`.
    """

    def __init__(
        self,
        provider: BaseProvider,
        contract_address: str,
        validator: IValidatorValidator = None,
    ):
        """Get an instance of wrapper for smart contract.

        :param provider: instance of :class:`web3.providers.base.BaseProvider`
        :param contract_address: where the contract has been deployed
        :param validator: for validation of method inputs.
        """
        self.contract_address = contract_address

        if not validator:
            validator = IValidatorValidator(provider, contract_address)

        self._web3_eth = Web3(  # type: ignore # pylint: disable=no-member
            provider
        ).eth

        functions = self._web3_eth.contract(
            address=to_checksum_address(contract_address), abi=IValidator.abi()
        ).functions

        self.is_valid_signature = IsValidSignatureMethod(
            provider, contract_address, functions.isValidSignature, validator
        )

    @staticmethod
    def abi():
        """Return the ABI to the underlying contract."""
        return json.loads(
            '[{"constant":true,"inputs":[{"internalType":"bytes32","name":"hash","type":"bytes32"},{"internalType":"address","name":"signerAddress","type":"address"},{"internalType":"bytes","name":"signature","type":"bytes"}],"name":"isValidSignature","outputs":[{"internalType":"bytes4","name":"","type":"bytes4"}],"payable":false,"stateMutability":"view","type":"function"}]'  # noqa: E501 (line-too-long)
        )


# pylint: disable=too-many-lines
